package org.apache.myfaces.custom.convertboolean;

import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.webapp.ConverterTag;
import javax.faces.webapp.UIComponentTag;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

/**
 * A tag that invokes the {@link BooleanConverter} and lets the developer
 * specify the desired values for a boolean true or false.
 * <p>
 * Example:
 * <code>
 * <h:outputText value="#{backingBean.customer.enjoysJazz}">
 *     <t:convertBoolean trueValue="Yes" falseValue="No"/>
 * </h:outputText>
 * </code>
 * <p>
 * @author Ken Weiner
 */
public class ConvertBooleanTag extends ConverterTag {
    private String trueValue;
    private String falseValue;

    public ConvertBooleanTag() {
        setConverterId(BooleanConverter.CONVERTER_ID);
    }

    public String getFalseValue() {
        return falseValue;
    }

    public void setFalseValue(String falseValue) {
        this.falseValue = falseValue;
    }

    public String getTrueValue() {
        return trueValue;
    }

    public void setTrueValue(String trueValue) {
        this.trueValue = trueValue;
    }

    public void setPageContext(PageContext context) {
        super.setPageContext(context);
        setConverterId(BooleanConverter.CONVERTER_ID);
    }

    protected Converter createConverter() throws JspException {
        BooleanConverter converter = (BooleanConverter) super.createConverter();

        FacesContext facesContext = FacesContext.getCurrentInstance();
        setConverterTrueValue(facesContext, converter, this.trueValue);
        setConverterFalseValue(facesContext, converter, this.falseValue);

        return converter;
    }

    private static void setConverterTrueValue(FacesContext facesContext,
            BooleanConverter converter, String value) {
        if (value == null) {
            return;
        }
        if (UIComponentTag.isValueReference(value)) {
            ValueBinding vb = facesContext.getApplication().createValueBinding(value);
            converter.setTrueValue((String) vb.getValue(facesContext));
        } else {
            converter.setTrueValue(value);
        }
    }

    private static void setConverterFalseValue(FacesContext facesContext,
            BooleanConverter converter, String value) {
        if (value == null) {
            return;
        }
        if (UIComponentTag.isValueReference(value)) {
            ValueBinding vb = facesContext.getApplication().createValueBinding(value);
            converter.setFalseValue((String) vb.getValue(facesContext));
        } else {
            converter.setFalseValue(value);
        }
    }

}
