/*
 * Copyright 2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.myfaces.custom.graphicimageajax;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.myfaces.taglib.html.ext.HtmlGraphicImageTag;

/**
 * @author Sylvain Vieujot
 * @version $Revision: 292080 $ $Date: $
 */

public class GraphicImageAjaxTag extends HtmlGraphicImageTag
{
    private static Log log = LogFactory.getLog(GraphicImageAjaxTag.class);

    private String _getContentTypeMethodProperty;
    private String _getBytesMethodProperty;

    public String getComponentType() {
        return GraphicImageAjax.COMPONENT_TYPE;
    }

    public String getRendererType() {
        return GraphicImageAjaxRenderer.RENDERER_TYPE;
    }

    public void release() {
        super.release();
        _getContentTypeMethodProperty = null;
        _getBytesMethodProperty = null;
    }

    protected void setProperties(UIComponent component) {
        super.setProperties(component);        
        
        setGetContentTypeMethodProperty(getFacesContext(), component, _getContentTypeMethodProperty);
        setGetBytesMethodProperty(getFacesContext(), component, _getBytesMethodProperty);
    }

    protected void setGetContentTypeMethodProperty(FacesContext context,
                                                 UIComponent component,
                                                 String getContentTypeMethod)
    {
        if (getContentTypeMethod != null)
        {
            if (!(component instanceof GraphicImageAjax))
            {
                throw new IllegalArgumentException("Component " + component.getClientId(context) + " is no GraphicImageAjax");
            }
            if (isValueReference(getContentTypeMethod))
            {
            	MethodBinding mb = context.getApplication().createMethodBinding(getContentTypeMethod,new Class[]{});
                ((GraphicImageAjax)component).setGetContentTypeMethod(mb);
            }
            else
            {
                log.error("Invalid expression " + getContentTypeMethod);
            }
        }
    }

    protected void setGetBytesMethodProperty(FacesContext context,
            UIComponent component,
            String getBytesMethod)
	{
    	if (getBytesMethod != null)
        {
            if (!(component instanceof GraphicImageAjax))
            {
                throw new IllegalArgumentException("Component " + component.getClientId(context) + " is no GraphicImageAjax");
            }
            if (isValueReference(getBytesMethod))
            {
            	MethodBinding mb = context.getApplication().createMethodBinding(getBytesMethod,new Class[]{});
                ((GraphicImageAjax)component).setGetBytesMethod(mb);
            }
            else
            {
                log.error("Invalid expression " + getBytesMethod);
            }
        }
	}
    
    public void setGetContentTypeMethod(String getContentTypeMethodProperty)
    {
    	this._getContentTypeMethodProperty = getContentTypeMethodProperty;
    }
    
    public void setGetBytesMethod(String getBytesMethodProperty)
    {
    	this._getBytesMethodProperty = getBytesMethodProperty;
    }
}