/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.myfaces.custom.inputsuggestajax;

import org.apache.myfaces.component.html.ext.HtmlInputText;
import org.apache.myfaces.custom.ajax.api.AjaxComponent;
import org.apache.myfaces.custom.ajax.api.AjaxRenderer;

import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.render.Renderer;
import java.io.IOException;

/**
 * @author Gerald Muellan (latest modification by $Author: svieujot $)
 * @author Martin Marinschek
 *
 * @version $Revision: 169662 $ $Date: 2005-05-11 19:57:24 +0200 (Wed, 11 May 2005) $
 */

public class InputSuggestAjax extends HtmlInputText implements AjaxComponent
{
    private MethodBinding _suggestedItemsMethod;
    private Integer _maxSuggestedItems;

    private String _popupId;
    private String _popupStyleClass;
    private String _popupStyle;

    private String _listId;
    private String _listStyleClass;
    private String _listStyle;

    private String _listItemStyleClass;
    private String _listItemStyle;

    private String _layout;

    public Object saveState(FacesContext context)
    {
        Object[] values = new Object[12];
        values[0] = super.saveState(context);
        values[1] = saveAttachedState(context, _suggestedItemsMethod);
        values[2] = _popupId;
        values[3] = _popupStyleClass;
        values[4] = _popupStyle;
        values[5] = _listId;
        values[6] = _listStyleClass;
        values[7] = _listStyle;
        values[8] = _listItemStyleClass;
        values[9] = _listItemStyle;
        values[10] = _layout;
        values[11] = _maxSuggestedItems;

        return values;
    }

    public void restoreState(FacesContext context, Object state)
    {
        Object values[] = (Object[])state;
        super.restoreState(context, values[0]);
        _suggestedItemsMethod = (MethodBinding) restoreAttachedState(context, values[1]);
         _popupId = (String) values[2];
        _popupStyleClass = (String) values[3];
        _popupStyle = (String) values[4];
        _listId = (String) values[5];
        _listStyleClass = (String) values[6];
        _listStyle = (String) values[7];
        _listItemStyleClass = (String) values[8] ;
        _listItemStyle = (String) values[9];
        _layout = (String) values[10];
        _maxSuggestedItems = (Integer) values[11];
    }

    public void encodeAjax(FacesContext context)
            throws IOException
    {
        if (context == null) throw new NullPointerException("context");
        if (!isRendered()) return;
        Renderer renderer = getRenderer(context);
        if (renderer != null && renderer instanceof AjaxRenderer)
        {
            ((AjaxRenderer) renderer).encodeAjax(context, this);
        }
    }

    public String getLayout()
    {
       if (_layout != null) return _layout;
       return "default";
    }

    public void setLayout(String layout)
    {
        _layout = layout;
    }

     public void setSuggestedItemsMethod(MethodBinding suggestedItemsMethod)
    {
       _suggestedItemsMethod = suggestedItemsMethod;
    }

    public MethodBinding getSuggestedItemsMethod()
    {
        return _suggestedItemsMethod;
    }

    public String getPopupId()
    {
        return _popupId;
    }

    public void setPopupId(String popupId)
    {
        _popupId = popupId;
    }

    public String getPopupStyleClass()
    {
        return _popupStyleClass;
    }

    public void setPopupStyleClass(String popupStyleClass)
    {
        _popupStyleClass = popupStyleClass;
    }

    public String getPopupStyle()
    {
        return _popupStyle;
    }

    public void setPopupStyle(String popupStyle)
    {
        _popupStyle = popupStyle;
    }

    public String getListId()
    {
        return _listId;
    }

    public void setListId(String listId)
    {
        _listId = listId;
    }

    public String getListStyleClass()
    {
        return _listStyleClass;
    }

    public void setListStyleClass(String listStyleClass)
    {
        _listStyleClass = listStyleClass;
    }

    public String getListStyle()
    {
        return _listStyle;
    }

    public void setListStyle(String listStyle)
    {
        _listStyle = listStyle;
    }

    public String getListItemStyleClass()
    {
        return _listItemStyleClass;
    }

    public void setListItemStyleClass(String listItemStyleClass)
    {
        _listItemStyleClass = listItemStyleClass;
    }

    public String getListItemStyle()
    {
        return _listItemStyle;
    }

    public void setListItemStyle(String listItemStyle)
    {
        _listItemStyle = listItemStyle;
    }

	public Integer getMaxSuggestedItems() {
		return _maxSuggestedItems;
	}

	public void setMaxSuggestedItems(Integer suggestedItems) {
		_maxSuggestedItems = suggestedItems;
	}
}
